@@ -6,6 +6,7 @@ and classes until a better place in the distribution is found.
 # psycopg/extras.py - miscellaneous extra goodies for psycopg
 #
 # Copyright (C) 2003-2019 Federico Di Gregorio  <fog@debian.org>
+# Copyright (C) 2020 The Psycopg Team
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -124,12 +125,15 @@ class DictCursorBase(_cursor):
 class DictConnection(_connection):
     """A connection that uses `DictCursor` automatically."""
     def cursor(self, *args, **kwargs):
-        kwargs.setdefault('cursor_factory', DictCursor)
+        kwargs.setdefault('cursor_factory', self.cursor_factory or DictCursor)
         return super(DictConnection, self).cursor(*args, **kwargs)
 
 
 class DictCursor(DictCursorBase):
-    """A cursor that keeps a list of column name -> index mappings."""
+    """A cursor that keeps a list of column name -> index mappings__.
+
+    .. __: https://docs.python.org/glossary.html#term-mapping
+    """
 
     def __init__(self, *args, **kwargs):
         kwargs['row_factory'] = DictRow
@@ -195,6 +199,10 @@ class DictRow(list):
     def __contains__(self, x):
         return x in self._index
 
+    def __reduce__(self):
+        # this is apparently useless, but it fixes #1073
+        return super(DictRow, self).__reduce__()
+
     def __getstate__(self):
         return self[:], self._index.copy()
 
@@ -221,7 +229,7 @@ class DictRow(list):
 class RealDictConnection(_connection):
     """A connection that uses `RealDictCursor` automatically."""
     def cursor(self, *args, **kwargs):
-        kwargs.setdefault('cursor_factory', RealDictCursor)
+        kwargs.setdefault('cursor_factory', self.cursor_factory or RealDictCursor)
         return super(RealDictConnection, self).cursor(*args, **kwargs)
 
 
@@ -291,7 +299,7 @@ class RealDictRow(OrderedDict):
 class NamedTupleConnection(_connection):
     """A connection that uses `NamedTupleCursor` automatically."""
     def cursor(self, *args, **kwargs):
-        kwargs.setdefault('cursor_factory', NamedTupleCursor)
+        kwargs.setdefault('cursor_factory', self.cursor_factory or NamedTupleCursor)
         return super(NamedTupleConnection, self).cursor(*args, **kwargs)
 
 
@@ -391,6 +399,7 @@ class NamedTupleCursor(_cursor):
 def _cached_make_nt(cls, key):
     return cls._do_make_nt(key)
 
+
 # Exposed for testability, and if someone wants to monkeypatch to tweak
 # the cache size.
 NamedTupleCursor._cached_make_nt = classmethod(_cached_make_nt)
@@ -405,11 +414,12 @@ class LoggingConnection(_connection):
     def initialize(self, logobj):
         """Initialize the connection to log to `!logobj`.
 
-        The `!logobj` parameter can be an open file object or a Logger
+        The `!logobj` parameter can be an open file object or a Logger/LoggerAdapter
         instance from the standard logging module.
         """
         self._logobj = logobj
-        if _logging and isinstance(logobj, _logging.Logger):
+        if _logging and isinstance(
+                logobj, (_logging.Logger, _logging.LoggerAdapter)):
             self.log = self._logtologger
         else:
             self.log = self._logtofile
@@ -442,7 +452,7 @@ class LoggingConnection(_connection):
 
     def cursor(self, *args, **kwargs):
         self._check()
-        kwargs.setdefault('cursor_factory', LoggingCursor)
+        kwargs.setdefault('cursor_factory', self.cursor_factory or LoggingCursor)
         return super(LoggingConnection, self).cursor(*args, **kwargs)
 
 
@@ -485,7 +495,8 @@ class MinTimeLoggingConnection(LoggingConnection):
             return msg + _os.linesep + "  (execution time: %d ms)" % t
 
     def cursor(self, *args, **kwargs):
-        kwargs.setdefault('cursor_factory', MinTimeLoggingCursor)
+        kwargs.setdefault('cursor_factory',
+            self.cursor_factory or MinTimeLoggingCursor)
         return LoggingConnection.cursor(self, *args, **kwargs)
 
 
